import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';

import '../../generated/colors.gen.dart';

const _durationProgressingInMillis = Duration(milliseconds: 300);

class KntRectangleProgress extends StatefulWidget {
  final double? progress;
  final double? width;
  final double? height;
  final double? heightFactor;
  final Color? color;
  final Color? backgroundColor;
  final double? borderRadius;
  final Duration? duration;

  const KntRectangleProgress({
    Key? key,
    this.progress,
    this.width,
    this.height,
    this.heightFactor,
    this.color,
    this.backgroundColor,
    this.borderRadius,
    this.duration,
  }) : super(key: key);

  @override
  _KntRectangleProgressState createState() => _KntRectangleProgressState();
}

class _KntRectangleProgressState extends State<KntRectangleProgress>
    with SingleTickerProviderStateMixin {
  late AnimationController _progressController;
  late Animation<double> _animation;
  late double _progress;
  late double _oldProgress;

  @override
  void initState() {
    super.initState();
    _prepareAnimation();
    _runProgress();
  }

  @override
  void didUpdateWidget(KntRectangleProgress oldWidget) {
    super.didUpdateWidget(oldWidget);
    _runProgress();
  }

  @override
  void dispose() {
    _progressController.dispose();
    super.dispose();
  }

  void _prepareAnimation() {
    _progress = widget.progress ?? 0.0;
    _oldProgress = _progress;
    _progressController = AnimationController(
      vsync: this,
      duration: widget.duration ?? _durationProgressingInMillis,
    )..addListener(() => setState(() {}));
    _animation = CurvedAnimation(
      parent: _progressController,
      curve: Curves.ease,
    );
  }

  void _runProgress() {
    // Current progress becomes the old ones
    _oldProgress = _progress;
    // Update to the latest progress
    _progress = widget.progress ?? 0.0;
    if (_oldProgress == _progress) {
      return;
    }
    // Always start animation from the beginning
    if (_progressController.isAnimating || _progressController.isCompleted) {
      _progressController.reset();
    }
    _progressController.forward();
  }

  @override
  Widget build(BuildContext context) {
    final height = widget.height ?? 4.w;
    final heightFactor = widget.heightFactor ?? 1.0;
    final borderRadius = widget.borderRadius ?? 0.w;
    return Container(
      width: widget.width ?? 1.sw,
      height: height,
      decoration: BoxDecoration(
        color: widget.backgroundColor ?? ColorName.dustyGray,
        borderRadius: BorderRadius.circular(borderRadius),
      ),
      child: FractionallySizedBox(
        widthFactor:
            _oldProgress + (_progress - _oldProgress) * _animation.value,
        heightFactor: heightFactor,
        alignment: Alignment.centerLeft,
        child: Container(
          height: height * heightFactor,
          decoration: BoxDecoration(
            color: widget.color ?? ColorName.jungleGreen,
            borderRadius: BorderRadius.circular(borderRadius * heightFactor),
          ),
        ),
      ),
    );
  }
}
